<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\CMP\Util\CmpUtilities;

class Cmp07600AlteracaoAprovacaoAfController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $alteracaoAprovacaoAfService;
    protected $cmpUtilities;
    protected $fechamentoAF;
    
    public function __construct ($crudService, $parameterBag, $alteracaoAprovacaoAfService, CmpUtilities $cmpUtilities, $fechamentoAF) {
        
        $this->alteracaoAprovacaoAfService = $alteracaoAprovacaoAfService;
        $this->cmpUtilities = $cmpUtilities;
        $this->fechamentoAF = $fechamentoAF;
        
        parent::__construct($crudService, $parameterBag);
        
    }
    
    public function parseFloat($floatVar) {
        if(is_string($floatVar)) {
            $floatVar = str_replace(".", "", $floatVar);    
            $floatVar = str_replace(",", ".", $floatVar);
        }
        return $floatVar;
    }
    
    public function getSystemParamGeral(DTO\Request $row, DTO\Response $response){
        
        $paramGeral = $this->alteracaoAprovacaoAfService->getSystemParamGeral();
        $response->addDataSet(new DataSet("PARAMGERAL", $paramGeral));
        
    }
    
    public function getAfInfos(DTO\Request $row, DTO\Response $response){
        
        $row = $row->getRow();
        $infos = array();
        
        $branchOperator = self::getBranchOperator($row);
        $infos["BRANCH_OPERATOR"] = $branchOperator;
        
        $row["CDFILCENTCMP"] = self::getTradingCenter($row);
        $infos["TRADING_CENTER"] = $row["CDFILCENTCMP"];
        
        $permissions = self::getPermissions($row);
        $infos["PERMISSIONS"] = $permissions;
        
        $integration = self::getIntegration($row);
        $infos["INTEGRATION"] = $integration;
        
        $frequency = self::getFrequency($row);
        $infos["FREQUENCY"] = $frequency;
        
        $response->addDataSet(new DataSet("AF_INFOS", array($infos)));
        
    }
    
    public function getBranchOperator($row){
        
        $selectParams = array(
            "P_CDFILDEST" => $row["CDFILDEST"]
        );
        
        $operators = $this->alteracaoAprovacaoAfService->getBranchOperator($selectParams);
        return $operators;
        
    }
    
    public function getTradingCenter($row) {
        $selectParams = array(
            "P_CDFILDEST" => $row["CDFILDEST"]
        );
        
        $tradingCenter = $this->alteracaoAprovacaoAfService->getTradingCenter($selectParams);
        
        return $tradingCenter[0]["CDFILCENTCMP"];
    }
    
    public function getPermissions($row){
        $selectParams = array(
            "P_CDFILCENTCMP" => $row["CDFILCENTCMP"]
        );
        $permissions = $this->alteracaoAprovacaoAfService->getPermissions($selectParams);
        return $permissions;
        
    }
    
    public function getIntegration($row){
        
        $selectParams = array(
            "P_CDFILDEST"   => $row["CDFILDEST"],
            "P_NRAUTOFORN"  => $row["NRAUTOFORN"]
        );
        
        $integration = $this->alteracaoAprovacaoAfService->getIntegration($row, $selectParams);
        return $integration;
        
    }
    
    public function getFrequency($row){
        
        $selectParams = array(
            "P_CDFILDEST"     => $row["CDFILDEST"],
            "P_CDFORNECED"  => $row["CDFORNECED"]
        );
        
        $frequency = $this->alteracaoAprovacaoAfService->getFrequency($selectParams);
        return $frequency;
        
    }
    
    public function executeProcedureCarregaPeriforn(DTO\Request $request, DTO\Response $response){
      
        $row = $request->getRow();
        $returnedDate = $this->alteracaoAprovacaoAfService->executeProcedureCarregaPeriforn($row);
        
        $response->addDataSet(new DataSet("DATA", array("RETURNED_DATE" => $returnedDate)));
        
    }
    
    public function sendEmailToProductResponsibleOperators(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $responsilbleOperators = self::getAfResponsibleOperators($row);
        
        if (empty($responsilbleOperators)) {
            $msg = 'Impossível enviar e-mail, não há operador responsável por esta unidade.';
            $response->setError(new Error($msg, 1));
        }
        
        $row["OPERADORES_RESPONSAVEIS"] = $responsilbleOperators;
        
        $this->alteracaoAprovacaoAfService->sendEmailToProductResponsibleOperators($row);
        
    }
    
    public function getAfResponsibleOperators($row){
        
        $selectParams = array(
            "P_CDFILDEST"     => $row["CDFILDEST"]
        );
        
        $operators = $this->alteracaoAprovacaoAfService->getAfResponsibleOperators($selectParams);
        return $operators;
        
    }
    
    public function updateAllAfProductDeliveryDate(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $updateParams = array(
            
            "P_CDFILDEST"   => $row["CDFILDEST"],
            "P_CDFILLANC"   => $row["CDFILLANC"],
            "P_NRAUTOFORN"  => $row["NRAUTOFORN"],
            "P_DTNOVAENTRAF"  => $row["DTNOVAENTRAF"]
            
        );
        
        $this->alteracaoAprovacaoAfService->updateAllAfProductDeliveryDate($updateParams);
        
    }
    
    public function getProductsWithNewDeliveryDate(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $selectParams = array(
            
            "P_CDFILDEST"   => $row["CDFILDEST"],
            "P_CDFILLANC"   => $row["CDFILLANC"],
            "P_NRAUTOFORN"  => $row["NRAUTOFORN"]
            
        );
        
        $products = $this->alteracaoAprovacaoAfService->getProductsWithNewDeliveryDate($selectParams);
        $response->addDataSet(new DataSet("PRODUCTS", $products));
    }
    
    public function updateInconsistencyAF(DTO\Request $request, DTO\Response $response) {
        $limiteParam   = $request->getRow()[0];
        $inconsisParam = $request->getRow()[1];
        $dadosAF       = $request->getRow()[2];
            
        try {
            $parcmpsup     = $this->alteracaoAprovacaoAfService->getParcmpsup(["CDFILIAL" => $dadosAF["CDFILDEST"], "NRORG" => $dadosAF["NRORG"]])[0];
            
            if(sizeof($limiteParam) > 0) {
                $dadosAF["IDAFFORALIM"] = "S";
                $dadosAF["CDGERENTE"]   = ", CDGERENTE = ''";
            } else {
                $dadosAF["IDAFFORALIM"] = "N";
                $dadosAF["CDGERENTE"]   = "";
            }
            
            if(sizeof($inconsisParam) > 0 || $parcmpsup["IDOBLIBAFMAN"] == "S") {
                $dadosAF["IDSITUATUAAF"] = "1";
            } else {
                $dadosAF["IDSITUATUAAF"] = "";
            }
            
            // Salva AF
            $this->alteracaoAprovacaoAfService->updateAf($dadosAF);
            
            // Atualiza Contar a Pagar
            $tpctpagar = $this->alteracaoAprovacaoAfService->updateCdtpctpagar($dadosAF);
            
            // Procedure CUSTO_AF
            $dadosAF["NRSEQAPURAF"] = null;
            $this->alteracaoAprovacaoAfService->CUSTO_AF($dadosAF);
            
            $response->addMessage(new Message("Success"));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function insertTaxes(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $autoforne = $taxes = $this->getAutoforne($row);
            $row['CDEMPRESA'] = $autoforne[0]['CDEMPRESA'];
            
            $taxes = $this->checkTaxes($row);
            
            if(sizeof($taxes) > 0) {
                $row["CDINSCESTA"] = $taxes[0]["CDINSCESTA"];
                $this->alteracaoAprovacaoAfService->IMPOSTO_ITEMAF($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function getCentralCompras(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $centralDeCompras = $this->cmpUtilities->getCentralCompras();
        
        $response->addDataSet(new DataSet("res", $centralDeCompras));
    }
    
    public function checkTaxes($params) {
        $conditions = array(
            ["columnName" => "CDFILDEST",   "operator" => "=",    "value" => $params["CDFILDEST"]],
            ["columnName" => "CDEMPRESA",   "operator" => "=",    "value" => $params["CDEMPRESA"]],
            ["columnName" => "NRORG",       "operator" => "=",    "value" => $params["NRORG"]    ]
        );
        $taxes = $this->crudService->getFilteredDataSet("cmp03900_verificaImposto", $conditions)->getRows();
        return $taxes;
    }
    
    public function getAutoforne($params) {
        $conditions = array(
            ["columnName" => "CDFILDEST",   "operator" => "=",    "value" => $params["CDFILDEST"]],
            ["columnName" => "CDFILLANC",   "operator" => "=",    "value" => $params["CDFILLANC"]],
            ["columnName" => "NRAUTOFORN",  "operator" => "=",    "value" => $params["NRAUTOFORN"]],
            ["columnName" => "NRORG",       "operator" => "=",    "value" => $params["NRORG"]]
        );
        
        $autoforne = $this->crudService->getFilteredDataSet("autoforne", $conditions)->getRows();
        return $autoforne;
    }
    
    public function getParamSupplier(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $params = array(
            
            "P_CDFILDEST"   => $row["CDFILDEST"],
            "P_NRAUTOFORN"   => $row["NRAUTOFORN"]
            
        );
        
        $paramsSupplier = $this->alteracaoAprovacaoAfService->getParamSupplier($params);
        $response->addDataSet(new DataSet("PARAM", $paramsSupplier));
    }
    
    public function approvalItemAf(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $getRow = $request->getRow();
            $rows = $getRow['ROWS'];
            $boAtuali = $getRow['ATUSOLI'];
            
            foreach($rows as $row) {
                $stIdOperacao = null;
                $msgJust = $row['JUSTIFICATIVA'];
                $tipo = $row['NMSITITEM'];
                    
                if($row['QTNOVAPROGAF'] != null && $row['DTNOVAENTRAF'] != null) {
                    if(self::parseFloat($row['QTNOVAPROGAF']) == 0) {
                        $stIdOperacao = 'E';
                    } else {
                        $stIdOperacao = 'A';
                    }
                } else if($row['DTNOVAENTRAF'] != null) {
                    $stIdOperacao = 'A';
                } else if($row['QTNOVAPROGAF'] != null) {
                    if(self::parseFloat($row['QTNOVAPROGAF']) == 0) {
                        $stIdOperacao = 'E';
                    } else {
                        $stIdOperacao = 'A';
                    }
                }
                if($boAtuali && $tipo == 'A') {
                    if($row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG']) {
                        (float)$dbQtSoliProd = self::parseFloat($row['QTNOVAPROGAF']) - self::parseFloat($row['QTORIG']);
                            
                        $adqEntrItmSol = $this->alteracaoAprovacaoAfService->adqEntrItmSol($row);
                            
                        $adqItemSoli = $this->alteracaoAprovacaoAfService->adqItemSoli($row, $adqEntrItmSol);
                            
                        if(count($adqItemSoli) > 0) {
                            foreach($adqItemSoli as $itmSol) {
                                (float)$dbQtSoliProd = $dbQtSoliProd * self::parseFloat($itmSol['VRFATOCONV']) / self::parseFloat($itmSol['VRFATOSOLI']);
                                    
                                if($dbQtSoliProd > 0) {
                                    $this->alteracaoAprovacaoAfService->acmUpQtSoliProd($itmSol, $dbQtSoliProd);
                                    $this->alteracaoAprovacaoAfService->acmInsMovOpSol($itmSol, $msgJust, (self::parseFloat($itmSol['QTSOLIPROD']) + $dbQtSoliProd));
                                        
                                    $dbQtSoliProd = 0;
                                }
                                if($dbQtSoliProd < 0) {
                                    if(self::parseFloat($itmSol['QTSOLIPROD']) > $dbQtSoliProd) {
                                        $this->alteracaoAprovacaoAfService->acmUpQtSoliProd($itmSol, $dbQtSoliProd);
                                        $this->alteracaoAprovacaoAfService->acmInsMovOpSol($itmSol, $msgJust, (self::parseFloat($itmSol['QTSOLIPROD']) + $dbQtSoliProd));
                                            
                                        $dbQtSoliProd = 0;
                                    } else {
                                        $this->alteracaoAprovacaoAfService->acmDelItemSoli($itmSol);
                                        $this->alteracaoAprovacaoAfService->acmInsMovOpSol($itmSol, $msgJust, 0);
                                            
                                        (float)$dbQtSoliProd = $dbQtSoliProd + self::parseFloat($itmSol['QTSOLIPROD']);
                                    }
                                }
                            }
                        }
                    }
                }
                if($row['DTNOVAENTRAF'] != null) {
                    if(substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10)) {
                        $adqAux_Imposto = $this->alteracaoAprovacaoAfService->adqAux_Imposto($row);
                            
                        if(count($adqAux_Imposto) > 0) {
                            if(self::parseFloat($adqAux_Imposto[0]['CTIMP']) > 0) {
                                $this->alteracaoAprovacaoAfService->acmDelImpItm($row);
                                    
                                if($stIdOperacao != 'E') {
                                    $adqExisteItemautf = $this->alteracaoAprovacaoAfService->adqExisteItemautf($row);
                                        
                                    $this->alteracaoAprovacaoAfService->acmDelItemSoliRelacAF($row);
                                        
                                    if(count($adqExisteItemautf) == 0) {
                                        $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, null);
                                    } else {
                                        $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, $adqExisteItemautf);
                                        
                                        $this->alteracaoAprovacaoAfService->acmDelItemAutF($row);
                                    }
                                }
                                
                                foreach($adqAux_Imposto as $imp) {
                                    $this->alteracaoAprovacaoAfService->acmInsImpItm($row, $imp);
                                }
                            } else {
                                if($stIdOperacao != 'E') {
                                    $adqExisteItemautf = $this->alteracaoAprovacaoAfService->adqExisteItemautf($row);
                                        
                                    $this->alteracaoAprovacaoAfService->acmDelItemSoliRelacAF($row);
                                        
                                    if(count($adqExisteItemautf) == 0) {
                                        $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, null);
                                    } else {
                                        $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, $adqExisteItemautf);
                                        
                                        $this->alteracaoAprovacaoAfService->acmDelItemAutF($row);
                                    }
                                }
                            }
                        } else {
                            if($stIdOperacao != 'E') {
                                $adqExisteItemautf = $this->alteracaoAprovacaoAfService->adqExisteItemautf($row);
                                        
                                $this->alteracaoAprovacaoAfService->acmDelItemSoliRelacAF($row);
                                        
                                if(count($adqExisteItemautf) == 0) {
                                    $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, null);
                                } else {
                                    $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, $adqExisteItemautf);
                                        
                                    $this->alteracaoAprovacaoAfService->acmDelItemAutF($row);
                                }
                            }
                        }
                    } else {
                        $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, null);
                    }
                } else {
                    $this->alteracaoAprovacaoAfService->acmUpdItm($row, $tipo, $msgJust, null);
                }
                if($row['QTNOVAPROGAF'] != null) {
                    if(self::parseFloat($row['QTNOVAPROGAF']) != self::parseFloat($row['QTORIG'])) {
                        $this->fechamentoAF->fechaAF($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], self::getTradingCenter($row));
                    }
                }
                if($row['QTNOVAPROGAF'] != null && self::parseFloat($row['QTNOVAPROGAF']) == 0) {
                    $this->alteracaoAprovacaoAfService->acmDelItemAutF($row);
                }
                if($tipo == 'R') {
                    if($row['QTNOVAPROGAF'] != null && self::parseFloat($row['QTNOVAPROGAF']) == 0) {
                        $this->alteracaoAprovacaoAfService->acmDelItemAutF($row);
                    }
                    $sqlWT = $this->alteracaoAprovacaoAfService->sqlWebTunning();
                    
                    $this->alteracaoAprovacaoAfService->acmInsMovOpAF($row, $sqlWT[0], $msgJust, $stIdOperacao);
                    $this->alteracaoAprovacaoAfService->acmInsLogPlanItemAF($row, $msgJust, $stIdOperacao);
                }
                if($msgJust != null || $msgJust != '' || !empty($msgJust)) {
                    $sqlWT = $this->alteracaoAprovacaoAfService->sqlWebTunning();
                    
                    $this->alteracaoAprovacaoAfService->acmInsMovOpAF1($row, $sqlWT[0], $msgJust, $stIdOperacao);
                    $this->alteracaoAprovacaoAfService->acmInsLogPlanItemAF1($row, $msgJust, $stIdOperacao);
                }
            }
                
            //TODO::envia email gestor
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
}